Module.define('ajaxutils',['jquery'],function($) {
	
	var options = { debounceTime:300 };
	
	var queues = {};
	
	function ajaxQ() {
		var head, tail, lastTime,
		    next = function() {
				var now = Date.now();
				if (now-lastTime < options.debounceTime) {
					setTimeout(next,options.debounceTime - (now-lastTime));
					return;
				}
				if (head.next) {
					head = head.next;
					head.xhr = $.ajax(head.settings).always(next);
				} else {
					head = tail = null;
				}
			};			
		return {
			add: function( settings ) {		
				var toAdd = { settings : settings };
				if (head == null) {					
					head = tail = toAdd;
					lastTime = Date.now();
					head.xhr = $.ajax(settings);
					head.xhr.always(next);
				} else {
					tail.next = toAdd; 
					tail = toAdd;
				}
			},
			clear: function() {
				if (head) {				
					var xhr = head.xhr;
					head=tail=null;
					xhr.abort();
				}
			}
		};		
	}
	
	$.ajaxq = function( queueName, settings ) {
		var queue = queues[queueName] ;
		if (settings) {
			if (!queue) queue = queues[queueName] = ajaxQ();
			queue.add(settings);									
		} else if (queue) {
			queue.cancel();
			delete queues[queueName];
		}
	};
	
	$.fn.smartLoad = function( url, params, callback ) {
		
		var selector, type, response,
			self = this,
			off = url.indexOf(" ");

		if ( off >= 0 ) {
			selector = url.slice( off );
			url = url.slice( 0, off );
		}

		if ( $.isFunction( params ) ) {
			callback = params;
			params = undefined;
		} else if ( params && typeof params === "object" ) {
			type = "POST";
		}

		if ( self.length > 0 ) {
			var args = {
				url: url,
				type: type || 'GET',
				dataType: "html",
				data: params
			};
			prev = self.data('smartload.xhr');
			if (prev) {
				if ((Date.now() - prev.time) < options.debounceTime && 
					eq(args, prev.args)) {					
					if (callback) {
						prev.xhr.always(function(jqXHR, status) {
							self.each( callback, response || [ jqXHR.responseText, status, jqXHR ] );
						});
					}
				} else {
					prev.xhr.abort();
				}
			}
			var xhr = $.ajax(args).done(function( responseText ) {
				response = arguments;
				self.html( selector ?
					$("<div>").append( $.parseHTML( responseText ) ).find( selector ) :
					responseText );
			}).always( function( jqXHR, status ) {
				self.removeData('smartload.xhr');
				if (callback) self.each( callback, response || [ jqXHR.responseText, status, jqXHR ] );
			});		
			
			if (window.VCOM) {
				xhr.fail(VCOM.errorHandler(function(){
					self.smartLoad(url, params, callback);
				}));
			}
			
			self.data('smartload.xhr',{xhr:xhr,args:args,time:Date.now()});
		}

		return this;
	};	

	$.fn.smartLoad.options = {
	   debounceTime : 300
	};

	
	function eq(a,b) {
		if (typeof a === 'object' && typeof b==='object') {
			for (var k in a) {
				if (!k in b) return false;
				if (!eq(a[k],b[k])) return false;
			}
			for (var k in b) {
				if (!k in b) return false;
			}
			return true;
		} else if (typeof a !== typeof b) {
			return false;
		} else {
			return a === b;
		}
	}
});

