Module.define('lru',function(){
	
	function LRU( options ) {		
		options = options || {};
		var ttl = options.ttl || (10 * 60 * 1000),
		    data = {},expires = {},
		    size=0, maxsize = options.maxsize,
		    timeout;
		
		function _size(toKeep) {
			if (arguments.length==1) { 
				if (toKeep==0) {
					data = {};
					used = {};
					size = 0;					
				} else {
					var keys = Object.keys(data);
					keys.sort(function(k1,k2){ return expires[k1]-expires[k2]; });
					for (var i=0;size > toKeep;i++) {
						delete data[keys[i]];
						delete expires[keys[i]];
						size--;
					}	
				}
				if (size ==0 && timeout != null) {
					window.clearTimeout(timeout);
					timeout = null;
				}				
			}			
			return size;
		}
		
		function purge() {
			if (timeout) window.clearTimeout(timeout);
			var keys = Object.keys(data);
			var now = Date.now();
			var lowmark = undefined;
			for (var i=keys.length-1;i>=0;i--) {
				var key = keys[i];
				var exp = expires[key];
				if (exp <= now) {
					delete data[key];
					delete expires[key];
					size--;
				} else if (lowmark == undefined || exp < lowmark) {
					lowmark = expires[key];
				}
			}
			if (maxsize != null && size > maxsize) {
				_size(maxsize)
			}
			if (lowmark) {
				timeout = setTimeout( purge, Math.max(1,now - lowmark) );
			}
		}
		
		function get(name) { 
			var exp = expires[name];
			if (expires[name]) {
				expires[name] = Date.now() + ttl;
				if (timeout==null) timeout = setTimeout( purge, ttl );
			}
			return data[name]; 
		}
		function set(name,value) {
			var ret = data[name] === undefined;
			data[name] = value;
			expires[name] = Date.now() + ttl;
			if (ret) size++;		
			if (timeout==null) timeout = setTimeout( purge, ttl );
			return ret;
		}
		function remove(name) {
			var exp = expires[name];				
			if (expires[name]) {
				delete data[name];
				delete expires[name];
				size--;
				return true;
			}
			return false;
		}
		
		return {
			get : get,
			set : set,
			put: set,
			remove : set,
			'delete' : remove,
			clear: function() { _size(0); },
			size: _size,			
			purge: purge
		};
	}
	
	return LRU;
	
});