Module = (function() {	
	(window.console || (window.console={'log':function() { }}));
	
	var head = document.getElementsByTagName("head")[0];	
	var modules = {};
	var deferred = [];	
	var loading = {};			
		
	function use( depends, dest ) {
		if (typeof depends == 'string') depends = [depends];	
		dest = dest || window;
		after(depends,function(){
			for (var i=0;i<arguments.length;i++) {
				dest[depends[i]] = arguments[i];
			}
		});
	}
	
	function after( depends, fn ) {
		if (typeof depends == 'string') depends = [depends];	
		if (!satisfied(depends)) {
			var def = function(){ after(depends,fn); };
			def.depends = depends;
			deferred.push(def);
			load.apply(window,depends);
		} else {
			var deps = (depends.length) ? depends.map(function(mod){ return modules[mod].data }) : depends;
			if (typeof fn === 'function') {
				fn.apply( window, deps );								
			}
		}		
	}
	
	function define(packageName, depends, definition ) {		
		if (modules[packageName]) return;
		if (arguments.length==2) {
			definition = depends;
			depends=[];
		}		
		var mod = modules[packageName] = {}; 
		after(depends,function() {
			var indata={};
			delete loading[packageName];
			mod.loaded=true;
			mod.data=definition.apply(indata,arguments) || indata;				
			runDeferred();
		});		
	}
			
	function load() {
		
		for (var i=0;i<arguments.length;i++) { 
			var mname = arguments[i];			
			if (!mname) continue;
			var type='js';
			if (mname.match(/\.css$/i)) {
				type='css';
			} else if (mname.match(/\.js$/i)) {
				mname=mname.substring(0,mname.length-3);
			} 
			var mod = modules[mname];
			if (mod == null && !loading[mname]) {
				loading[mname]=true;
				if (type=='js') {
					loadJs(getResource(Module.base+mname+".js"));
				} else if (type=='css') {
					loadCss(getResource(Module.base+mname),mname);				
				}
			}
		}
	}
			
	function runDeferred() {
		var toCall = [];
		for (var i=0;i<deferred.length;i++) {
			if (satisfied(deferred[i].depends)) {
				toCall.push(deferred[i]);				
				deferred.splice(i,1);
				i--;
			}		
		}
		for (var i=0;i<toCall.length;i++) {
			toCall[i]();
		}		
	}

	function satisfied(deps) {
		if (deps == null || deps.length==0) return true;
		for (var i=0;i<deps.length;i++) {
			if (!isLoaded(deps[i])) {
				return false;			
			}
		}		
		return true;
	}
	
	
	function isLoaded( mod ) {
		return (modules[mod] && modules[mod].loaded == true);
	}
			
	function loadJs(src) {		
		var scriptObj = document.createElement("script");
		scriptObj.type = 'text/javascript'; //scriptObj.async = true;
		scriptObj.src=src;
        head.appendChild(scriptObj);					
	}
		
	function loadCss(src, mname ) {
		var mod = modules[mname] = {};
		var link = document.createElement("link");			
		link.type = 'text/css';
		link.rel = 'stylesheet';
		link.href = src;			
		link.media = 'all';
		
		link.addEventListener('load',function() {
			delete loading[mname];
			mod.loaded = true;	
			runDeferred();			
		});	
		head.appendChild(link);			
		mod.data = link;
	}
		
	function getResource( url ) {
		if (Module.resourceMap && url && url.length && url.charAt(0)=='/') {
			var parts = url.substring(1).split(/\/+/);
			var ptr = Module.resourceMap;
			for (var i=0;i<parts.length && ptr;i++) {
				ptr = ptr[parts[i]];				
			}
			if (typeof ptr == 'string') {
				var dot = url.lastIndexOf('.');
				if (dot) {
					url= Module.host + url.substring(0,dot) + '-' + ptr + url.substring(dot);
				} else {
					url= Module.host + url + '-' + ptr;
				}
			} else {
				if (window.debug) console.log('resource not found',url);
			}
		}		
		return url;
	}
	
	var ret = {
		'load':load,
		'define':define,
		'after': after,
		'getResource': getResource,
		'use' : use
	};
		
	try {
		var scripts = (!document.scripts) ? document.getElementsByTagName('script') : document.scripts;
		var script=scripts[scripts.length-1];		
		if (script && script.src) {			
			if (script.src.indexOf('//')==0) {
				// "//" URL -- using resource server
				var parts = script.src.split(/\/+/);
				ret.host = location.protocol+"//"+parts[0];
				parts.splice(0,1);
				parts.pop();
				ret.base = (parts.length) ? "/"+parts.join('/')+"/" : "/"; 	
			} else if (script.src.indexOf(':')!=-1) {
				// absolute URL -- using resource server
				var parts = script.src.split(/\/+/);
				ret.host = parts[0]+"//"+parts[1];
				parts.splice(0,2);
				parts.pop();
				ret.base = (parts.length) ? "/"+parts.join('/')+"/" : "/"; 	
			} else {
				// relative URL
				ret.host = document.location.protocol+"//"+document.location.host;
				var parts = script.src.split(/\/+/);
				parts.pop();
				ret.base = (parts.length) ? "/"+parts.join('/')+"/" : "/"; 				
			}
		}
	} catch (e) {
		ret.host='';
		ret.base='/js/';		
	}
	
	return ret;
	
})();

