Module.define('slider',['jquery'],function($){
	
	$.fn.slider = function(options) {			
		if (!this.data('slider')) {
			slider.call(this,options);
		}
		return this.data('slider');
	}
	
	function slider(options) {
		var element = $(this).css({overflow:'hidden'}).data('slider',{
			next: next,
			prev: prev,
			go: go,
			getIndex: getIndex
		});
		
		var wrapper = $('<div/>').css({position:'absolute',left:0,right:0,top:0,bottom:0,'transition':'all .3s'});
		var children = element.children();
		
		wrapper.appendTo(element).append(children);
		
		children.each(function(idx){
			var child=$(this).css({'transform':'translate3d('+(idx*100)+'%,0,0)',top:0,left:0,width:'100%'});
			if (idx == 0) {
				child.addClass('selected').css({position:'relative'});
			} else {
				child.css({position:'absolute'});
			}
		});
		
		function getIndex() {
			return children.index(wrapper.children('.selected'));
		}
			
		function go(idx) {
			if (idx >=0 && idx < children.length) {
				var newSlide = $(children[idx]);
				newSlide.addClass('selected').css({position:'relative'}).addClass('selected').siblings().css('position','absolute').removeClass('selected');
				wrapper.css({'transition-duration':'.3s',
					         'transform':'translate3d(-'+(idx*100)+'%,0,0)'
					         });		         
				element.trigger('slider-go',idx);
				return true;
			}
		}
		
		element.on('next',next);
		element.on('prev',prev);
		
		function next() {
			var ret = go(getIndex()+1);
			if (ret) element.trigger('slider-next');
			return ret;
		}
		function prev() {
			var ret = go(getIndex()-1);
			if (ret) element.trigger('slider-prev');
			return ret;
		}
		
		
		
		element.on('touchstart',function(evt) {
			var dragging = false;
			if (evt.originalEvent.touches) {
				if (evt.originalEvent.touches.length > 1) return;
				evt = evt.originalEvent.touches[0];
			}
			wrapper.css('transition-duration','0');
			var startPosition = getPosition(wrapper);
			
			var start = { x: evt.pageX, y: evt.pageY };
			var last = { x: start.x, y: start.y };
			
			$(window).on('touchmove',moved).one('touchend',end);
			
			function moved(evt) {				
				if (evt.originalEvent.touches&&evt.originalEvent.touches.length > 1) {
					return cancel();
				}
				var touchPoint = evt.originalEvent.touches ? evt.originalEvent.touches[0] : evt;
				var dx = touchPoint.pageX-last.x, dy = touchPoint.pageY-last.y;
				last.x = touchPoint.pageX; last.y = touchPoint.pageY;
				
				if (dragging || Math.abs(dx) > Math.abs(dy)) {
					if (!dragging) dragging = true;	
					wrapper.css('transform','translate3d('+ (getPosition(wrapper).x+dx) +'px,0,0)');
					return false;
				} else {
					cancel();
				}
			}
			function cancel() {					
				$(window).off('touchmove',moved).off('touchend',end);					
				if (dragging) {
					dragging = false;
					go(getIndex());
				}
				
			}
			function end(evt) {
				$(window).off('touchmove',moved);
				if (dragging) {
					dragging = false;
					var touchPoint = evt.originalEvent.changedTouches ? evt.originalEvent.changedTouches[0] : evt;				
					var dx = touchPoint.pageX-start.x, dy = touchPoint.pageY-start.y;
					if (Math.abs(dx) > 30) {
						if ( ((dx>0) ? prev() : next()) ) {							
							return;
						}
					}
				} 
				go(getIndex());			
			}				
		});
	}
	
	function getPosition(el) {
		var matrix = el.css('-webkit-transform');
		matrix =matrix.split(')')[0].split(', ');
		return {x: +(matrix[12] || matrix[4]),
			    y: +(matrix[13] || matrix[5]) };
	}
	

	
});