Module.define('template',['jquery'], function($) {
	var module = this;
	var templateCache = {}; 
	
	this.clearCache = function() {  templateCache = {}; };
	this.load = load;
	
	$.fn.template = function( url, data, callback, onerror ) {
		var $this = this;
		load(url,
			 function(template){
				$this.each(function(){	
					try {
						$(this).html(template(data));
						if (typeof callback==='function') callback.call(this);
					} catch (e) {
						$this.empty();
						console.error(e);
						 ensureFunction( onerror ).call(this,e);
					}
				});
			},
			function(error) {
				$this.empty();
				onerror=ensureFunction(onerror);
				$this.each(function(){ onerror.call(this,error); });
			});
		return this;
	};
	
	function load(url,onload,onerror) {
		onload = ensureFunction(onload);
		onerror = ensureFunction( onerror );
		if (templateCache[url]) onload(templateCache[url]);
		else {
			$.get(url,function(text) {		
			try {
				 onload(templateCache[url] = compile(text));
			} catch (e) {
				if (console && console.error) console.error(e);
				onerror(e);
			}
			},'text').fail(function(x,s,e){ onerror(e); });
		}
	}
	
	function noop() {};
	
	function ensureFunction(f) {
		return (typeof f === 'function') ? f : noop
	}

	function compile( templateBody ) {
		var $$stringPool = [];
		var code = [ "var $$this=this, $$OUT=[]; if (!$$VIEW) $$VIEW={}; with($$VIEW) {" ];		
		var parts = split( templateBody );
		for (var i=0;i<parts.length;i++) {
			var part = parts[i];
			if (part == '<%') {
				if (parts.length < i+2) eofError(this);
				if (parts[i+2] != '%>') balanceError(this,part);	
				code.push(parts[i+1]);
				i+=2;				
			} else if (part == '<%=') {
				if (parts.length < i+2) eofError(this);
				if (parts[i+2] != '%>') balanceError(this,part+" "+parts[i+2]);				
				code.push('$$OUT.push($$this.escapeXML('+parts[i+1]+'));');				
				i+=2;
			} else if (part == '<%==') {
				if (parts.length < i+2) eofError(this);
				if (parts[i+2] != '%>') balanceError(this,part+" "+parts[i+2]);				
				code.push("$$OUT.push("+parts[i+1]+");");
				i+=2;			
			} else if (part == '<%--') {
				if (parts.length < i+2) eofError(this);
				if (parts[i+2] != '--%>') balanceError(this,part+" "+parts[i+2]);
				i+=2;		
			} else {				
				code.push("$$OUT.push($$this.$$stringPool["+$$stringPool.length+"]);");
				$$stringPool.push(part);
			}
		}
		code.push("}"); //END WITH
		code.push("return $$OUT.join('');");	
		var render = new Function( "$$VIEW", code.join("\n"));
		var constants = { '$$stringPool': $$stringPool, 'escapeXML': escapeXML };
		return function() { return render.apply( constants, arguments); }
	};
	
		
	function eofError(template) {
		throw new Error("invalid template, unexpected end of template");
	}
	
	function balanceError(template,tag) {
		throw new Error("invalid template, unbalanced tag: "+tag);
	}	
	
	function escapeXML(str) {
		if (str == null) return str;		
		return (''+str).replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;").replace(/'/gm, "&#39;"); 
	}
	
	var regex = new RegExp('(<%={0,2})|(%>)|(<%\!--)|(--%>)');
	
	function split(string) {
        var ret = [];
        for (var result;(result = regex.exec(string)) != null;) {
            if (result.index != 0) {
                ret.push(string.substring(0,result.index));
                string = string.slice(result.index);
            }               
            ret.push(result[0]);
            string = string.slice(result[0].length);                
        }
        if (string.length) ret.push(string);
        return ret;
	};	
	
});

