Module.define('vcom/autocomplete',['jquery'],function( $ ) {
	
	window.Autocomplete	= Autocomplete;
			
	$.fn.autocomplete = function() {		
		this.find('input[type=text]').each(function() {
			var resultsWindow = $('<div>').addClass('autocompleteResults').css({position:'absolute'});
			if (!this.__autocomplete) {
				var input = $(this)
							.attr('autocomplete','off')
							.attr('autocapitalize','off');
				var autocomplete = new Autocomplete(resultsWindow,this);
				this.__autocomplete = autocomplete;				
				resultsWindow.on('autocomplete.update', function(p) { 		
					if (autocomplete.hasWords()) {						
						var pos = input.offset();
						pos.top += input.outerHeight()+2;
						pos.width = input.outerWidth();
						resultsWindow.css(pos);					
						resultsWindow.show();
					} else {
						resultsWindow.hide();
					}				
				});					
				resultsWindow.on('autocomplete.click autocomplete.enter',function(word) {
					if (word) input.val( word.word );
					resultsWindow.hide();
				});
			}
			resultsWindow.appendTo(document.body).hide();			
		});		
	}
	
	Autocomplete.serviceURL = VCOM.server+'/autocomplete'
		
	function Autocomplete(resultsPane,searchField) {	
		
		resultsPane = $(resultsPane).data('vcom.autocomplete',this);
		searchField = $(searchField).data('vcom.autocomplete',this)
						.on('keydown',keyDown)
						.on('input',function(evt) {
							if (selectedIndex != -1) {
								selectItem(-1,true);
							}
							updateSoon(350);
						});
		var _this = this;
		var hasMore=true;
		var lastRequest;
		var selectedIndex = -1;
		var scrolling=false;	
		var timeout = null;			
		var words= resultsPane.find('li');		
		
		if (words.length) {
			hasMore = false;
			setTimeout(function(){ selectItem(0); },1);
		}
		
		this.setMonitorField = function(field) {
			field = $(field);
			if (searchField[0] != field[0]) {
				if (searchField != null) {
					$(searchField).off('keydown',keyDown).off('keyup',keyUp);
				}				
				clear();
				searchField = field;
				lastPost=null;
				searchField.on('keydown',keyDown).on('keyup',keyUp);
		    }
		}	
		
		
					
		this.setText=function( text, quiet ) {
			searchField.val(text||'');
			update({search:text},quiet);			
		}
		this.getText=function() {
			return searchField ? searchField.val().trim() : null;
		}
		this.hasWords=function() {
			return resultsPane.find('li').length > 0
		}
		this.isComplex=function() {
			return searchField.val().match(Autocomplete.complex);
		}
		
		
		function hasParent(parent,ele) {
			for (;ele != null;ele=ele.parentNode) {
				if (ele==parent) return true;					
			}
			return false
		}
								
		function keyDown(e) {
			var code = (e.keyCode) ? e.keyCode: e.which;
		    if (code == 40) {
		    	e.preventDefault();
		    	selectItem(selectedIndex+1); //down arrow				    	
		      	return false;		      
		    } else if (code == 38) {
		    	e.preventDefault();
		    	selectItem(selectedIndex-1); //up arrow			    		    	
		      	return false;
		    } else if (code == 39) {
		    	if (selectedIndex != -1) {
		    		var text=getSelectedWord().word;
		    		searchField.val(text);
					update({search:text});					
			    	e.preventDefault();
			    	return false;
		    	}				    
		    } else if (code == 33) {
		    	e.preventDefault();			    	
		    	selectItem(selectedIndex+Math.max(0,selectedIndex-4)-selectedIndex); //page up	
		    	return false;	
		    } else if (code == 34) {
		    	e.preventDefault();			    	
		    	selectItem(selectedIndex+(selectedIndex == -1 ? 1 : 4)); //page down	
		    	return false;	
		    } else if (code == 13) {
		    	e.preventDefault();	
		    	var evt = $.Event('autocomplete.enter',getSelectedWord());		    	
		    	resultsPane.trigger(evt);
		    	if (!evt.isDefaultPrevented()) {
		    		update({search:searchField.val(),spellingOnly:true});	
		    	}
		    	
		    	return false;	
		    }  
		}
		
		this.next = function(quiet) {
			selectItem(selectedIndex+1,quiet);
			return getSelectedWord();
		};
		this.prev = function(quiet) {
			selectItem(selectedIndex-1,quiet);
			return getSelectedWord();
		};
		this.peek = function(offset) {
			return getWordAt(selectedIndex+offset);
		};
		this.selectedIndex = function() {
			if (!arguments.length) return selectedIndex;
			selectItem(parseInt(arguments[0]),arguments[1]);
			return this;
		}
		
		this.indexOf = function (word,lang) {
			if (!lang) lang = 'en';
			for (var i=0;i<words.length;i++) {
				var $word = $(words[i]);
				if (word == $word.attr('word') && lang == $word.attr('lang')) {
					return i;
				}
			}
			return -1;
		}
		this.selectWord=function (word,lang,quiet) {
			return selectItem(this.indexOf(word,lang),quiet);			
		}
													
		var isControlCode = function(code) {
			return ((code == 13) || (code == 9) || 
				(code == 27) ||
		        (code >= 16 && code <= 20) ||
			    (code >= 33 && code <= 40)); 
		}
				
		function getWordAt(idx) {
			return toWordObject( $(words[idx]) );			
		}
		this.getWordAt = getWordAt;
		
		this.findWord=findWord;
		function findWord(word) {
			for (var i=0;i<words.length;i++) {
				var $word = $(words[i]);
				if (word == $word .attr('word')) {
					return toWordObject($word);
				}
			}
			return null;
		}
		
		function toWordObject(li) {
			return li.length ? {'word':li.attr('word'),
				 'lang':li.attr('lang'),
				 'synsetid':li.attr('synsetid'),
				 'definition': li.find('.definition').text(),
				 'learnable':li.hasClass('learnable'),
				  'freq':parseFloat(li.attr("freq"))} : undefined;	
		}
		
		function getSelectedWord() {
			return toWordObject(resultsPane.find('li.selected'));
		}		
		this.getSelectedWord = getSelectedWord;
		
		function selectItem(idx,quiet) {
			if (selectedIndex != idx) {	
				var lastIndex = selectedIndex;
				if (selectedIndex > -1) {
					$(words[selectedIndex]).removeClass('selected');	
				}
				if (idx>-1 && words.length) {		
					selectedIndex = Math.min(words.length-1,idx);												
					var item = $(words[selectedIndex]).addClass('selected');											
					if (!quiet) scrollItemIntoView(item);				
					if (hasMore && !scrolling && selectedIndex+5 > words.length ) {						
						fetchMore();
					}
				} else {
					selectedIndex = -1;
				}				
				if (!quiet) {
					var evt = $.Event('autocomplete.selectionchanged');
					evt.index = selectedIndex;
					evt.lastIndex = lastIndex;
					resultsPane.trigger(evt);
				}
				return true;
			}			
		}
		
		function clearSelection() {
			selectedIndex = -1;
			resultsPane.find('li.selected').removeClass('selected');		
			resultsPane.scrollTop(0); // todo need this?
		}
		this.clearSelection = clearSelection;
		
		this.scrollItemIntoView =scrollItemIntoView ;
		function scrollItemIntoView(selectedItem) {
			if (!selectedItem) selectedItem = resultsPane.find('li.selected');
			if (selectedItem.length) {
				var top = resultsPane.offset().top;  					
				var bottom = top + resultsPane.outerHeight();	
				
				var divtop =  selectedItem.offset().top;
				var divbottom =  divtop + selectedItem.outerHeight();
				
				if (divtop < top) {						
					resultsPane.scrollTop( resultsPane.scrollTop() + (divtop-top));
				} else if (divbottom > bottom) {
					resultsPane.scrollTop( resultsPane.scrollTop() + (divbottom-bottom));					
				}
				return true;
			}
		}
		
		function updateSoon( delay ) {
			cancel();			
			timeout = window.setTimeout(function(){ update({search:searchField.val()}) },delay);		
		}
		
		this.update = update;
		
		var lastParams = null;
		function update( params, quiet, callback ) {
			if (params && params.search) params.search = params.search.trim();
			
			if (equal(params,lastParams)) return false;			
			lastParams = params;
			if (!params || ((!params.search ||  params.search.length==0) && !params.listid)) {
				clear();					
				hasMore=false;					
				if (!quiet) resultsPane.trigger($.Event('autocomplete.update',params));	
				if (typeof callback ==='function') callback.call(_this,params);
			} else {	
				cancel();					
				lastRequest = 
					$.get(Autocomplete.serviceURL,params,function(results){	
						results = $(results);
						resultsPane.scrollTop(0);
						lastRequest = null;
						selectedIndex = -1;
						resultsPane.empty().append(results);				
						words= resultsPane.find('li');
						hasMore= (words.length < parseInt(results.data('total')));							
						if (!quiet) resultsPane.trigger($.Event('autocomplete.update',params));
						if (typeof callback ==='function') callback.call(_this,params);
					},'text')
					 .error(VCOM.errorHandler(function(){
						lastParams = null;
						update( params, quiet, callback );
					}));
			}
			return true;
		}

		resultsPane.on('scroll',function(evt) {
			if (hasMore && !scrolling &&
					this.scrollTop > 0 && 
					this.scrollHeight - (this.scrollTop + this.clientHeight) < 300  ) {					
				fetchMore();
			}
		});
		
		function copyParams(params) {
			var ret = {};
			for (var p in params) ret[p]=params[p];
			return ret;
		}
		
		function equal(p1,p2) {	
			if ((p1 && !p2) || (p2 && !p1)) return false 
			for (var p in p1) if (p1[p]!=p2[p]) return false;
			for (var p in p2) if (p1[p]!=p2[p]) return false;
			return true;
		}
		
		
		function fetchMore(callback) {
			cancel();
			var lastResult = resultsPane.find('ol:last-child');
			var params = copyParams(lastParams);
			params.startOffset=words.length;
			scrolling=true;								
			lastRequest = 
				$.get(Autocomplete.serviceURL,params,function(text){
					hasMore = ($(text).appendTo(resultsPane).length > 0);
					words = resultsPane.find('li');
					resultsPane.trigger($.Event('autocomplete.scroll',params));
			},'text').always(function(){
				scrolling = false;		
				if (callback) callback(_this);
			});
		}
		

		resultsPane.on('click','li', function(evt) {
			selectItem( $.inArray(this,words), true );
			scrollItemIntoView( $(this) );
			resultsPane.trigger($.Event('autocomplete.click',getSelectedWord()));			
			evt.stopPropagation();
			evt.preventDefault();
			return false;			
		});
		
		this.clear=clear;
		function clear() {
			cancel();
		 	try {
		 		searchField.val('');
		 		selectedIndex = -1;
		 		resultsPane.scrollTop( 0 ).empty();
		 	} catch (e) {}		 	
		}
		
		function cancel() {
			scrolling=false;
			if (timeout != null) {
				window.clearTimeout(timeout);
				timeout = null;
			}
			
			if (lastRequest != null) {
				lastRequest.abort();
				lastRequest = null;
			}
		}	
	}
	
	Autocomplete.complex = /[+:*?]/;	
	
	return { Autocomplete : Autocomplete };
});