Module.define('vcom/base', [ 'jquery', 
                             'backbone', 
                             'underscore', 'timezone',
                             'dotimeout','fastclick',
                             'gestures','ajaxutils','spinner',
                             ], function( $, Backbone, _, jsTz ) {
	
	window.android =  !!((/android/gi).test(window.navigator.appVersion));
	window.ios     =  !!((/iphone|ipad|ipod/gi).test(window.navigator.appVersion));
	
	// user timezone data!!
	var tz = null, 
        tzo = 0;
	
	// this gets appended to each ajax request
	var ajaxAppend = '';
	
	// show the ajax spinner after this amount of time
	var SLOW_AJAX_REQUEST = 750;
	var AJAX_TIMEOUT = 20000;
	
	// the main VCOM object
	var VCOM = new (Backbone.Model.extend({
		server    : 'http://app.vocabulary.com/app/1.0',
		sslServer : 'https://app.vocabulary.com/app/1.0',
		version   : '1.0.3',
		error     : globalError,
		networkError : networkError,
		storeURL     : (window.ios) ? 'itms-apps://itunes.apple.com/app/id815218412?at=10l6dK'
                       				: 'market://details?id=com.vocabulary.app',
		errorHandler : function(resume) { return function(xhr,status,error) { 
			if (status==='abort' || status==='canceled') {
				return; // request aborted, not an error
			}
			VCOM.networkError(xhr ? xhr.status : 0).then( resume || function(){} ); } 
		},	
		forceUpgrade : function () {
			var blocker = $('<div class="blocker network-error"><div class="ss-icon ss-downloadcloud"></div><h1></h1><button class="standard">Upgrade</button></div>');
			blocker.find('h1').text('This version of Vocabulary.com is no longer supported. Please update to the latest version.');
			blocker.find('button').on('click',function() { window.open(VCOM.storeURL,'_system'); });
			$(':focus').blur();
			$('body').addClass('withblocker').append(blocker);		
		},
		history   : {
			push: function() {},
			pop: function() {},
			remove: function() {}
		}
	}))();
	
	
	window.VCOM = VCOM;
	
	if (window.ios) Module.after('vcom/ios');
	else if (window.android) Module.after('vcom/android');	
	
	// set up native device listeners
	document.addEventListener('deviceready',deviceReady,false);
	document.addEventListener('resume',deviceResume,false);	
	document.addEventListener("offline",deviceOffline, false);
	document.addEventListener("online",deviceOnline, false);
	
	// when jquery is ready, set up some things
	$(function(){
		FastClick.attach(document.body);
	});
	
	function deviceResume() {
		updateTimezoneData();
	}	
	
	function deviceReady() {
		updateTimezoneData();
		configureAjax();
		// vibrate on incorrect challenge picks, if this device supports it
		if (navigator.notification && navigator.notification.vibrate) {		
			$(document).on('challenge.pick',function(e){
				if (!e.correct) navigator.notification.vibrate(100);
			});
		}				
	}

	function configureAjax() {
		$.ajaxSetup({ timeout: AJAX_TIMEOUT });
		
		// configure AJAX loader animation, prefilters
		var ajaxLoader = $('<div class="ajaxloading"></div>').css({
			'position':'fixed',
			'top': '50%',
			'left': '50%',
			'margin-left':'-32px',
			'margin-right':'-32px',
			'width': '64px',
			'height':'64px',
			'z-index':20000
		});	
		
		var longRequests = 0;
		$.ajaxPrefilter(function( options, _, jqXHR ) {
			if (typeof options.data === 'string') {
				options.data += ajaxAppend;
			}
			
			var timeout = setTimeout(function(){
					timeout = null;
					longRequests++;
					if (longRequests==1) ajaxLoader.appendTo('body').spin();				
				},SLOW_AJAX_REQUEST);
		    jqXHR.always( function() {
		    	if (timeout != null) {
		    		window.clearTimeout(timeout);
		    		timeout = null;
		    	} else {
		    		longRequests--;
		    		if (longRequests == 0) {
		    			ajaxLoader.spin(false).remove();		    			
		    		}		    		
		    	}
		    });
		});
	}
		
	function globalError(error) {
		if (typeof error == 'string') error = { message: error };
		if (window.debug) console.log('VCOM global error',error);
		VCOM.trigger('error',error);
	}
	
	function networkError(status) {
		status = (parseInt(status)||0);
		var ret = $.Deferred();
		if (status === 406) {
			VCOM.forceUpgrade();
		} else {
			var message = ((parseInt(status)||0) >= 500) ?
				"We\'re sorry, Vocabulary.com is having some server problems, please try again soon." : 
				"We\'re sorry, we seem to be having a problem contacting Vocabulary.com. Please check your connection and try again soon.";
			
			var blocker = $('<div class="blocker network-error"><div class="ss-icon ss-cloud"></div><h1></h1><button class="standard">Try Again</button></div>');
			blocker.find('h1').text(message);
			blocker.find('button').on('click',function() {
				var button = $(this);
				refreshAuth().done(function(auth){
					blocker.remove();
					resolveErrors();				
					if (auth && auth.loggedin) {
						ret.resolve();
					} else {
						VCOM.login();
					}
				}).fail(function() {
					button.addClass('shake').one('webkitAnimationEnd',function() { button.removeClass('shake'); });
				});
			});
			$(':focus').blur();
			$('body').addClass('withblocker').append(blocker);	
		}
		return ret;
	}

	function refreshAuth() {
		return $.post(VCOM.server+'/auth.json',{ secret: localStorage.getItem('challenge.secret') }).then(function(data){
			return data.auth;
		});
	}
	
	function deviceOffline() {
		$html = $('html');		
		if (!$html.hasClass('offline')) {
			$html.addClass('offline');
			//$('body').addClass('withblocker').append('<div class="blocker device-offline"><div class="ss-icon">cloud</div><h1>We\'re sorry, Vocabulary.com requires an active internet connection.</h1></div>');
		}
	}
	
	function deviceOnline() {
		$('html').removeClass('offline');
		$('body > .blocker button').trigger('click'); 
	}
	
	function resolveErrors() {
		if ($('body > .blocker').length==0) {
			$('body').removeClass('withblocker');
		}
	}
	
	function updateAjaxAppend() {
		if (tz != null)  ajaxAppend += '&tz='+encodeURIComponent(tz);
		if (tzo != null) ajaxAppend += '&tzo='+encodeURIComponent(tzo);
	    if (VCOM) {
	    	ajaxAppend += '&appver='+encodeURIComponent(VCOM.version);    
	    }
	    return ajaxAppend;
	}
	
	function updateTimezoneData() {
		tzo = -(new Date()).getTimezoneOffset();
		tz = jsTz.determine().name();
		 updateAjaxAppend();
		if (window.debug) console.log('user timezone',tz,tzo);
	}
	
	
	return VCOM;
});	
