Module.define('vcom/challenge',['jquery','vcom/base'],
function( $, VCOM ) {
	var pdata = null;

	var storage = {	
		reset : function() {
			storage.keys('challenge.').forEach(storage.deleteKey);
			storage.init();
		},
		keys: function (prefix) {
			var ret=[];
			for (var i=0;i<localStorage.length;i++) {
				var key =localStorage.key(i);
				if (key.indexOf(prefix) == 0) ret.push(key);					
			}
			return ret;
		},
		deleteKey: function(k) {
			window.localStorage.removeItem(k);
		},
		init : function () {
			window.localStorage.setItem('challenge.version', '1.0');				
		}		
	}
	
	VCOM.playerData = function() { 
		if (!arguments.length) return pdata;
		pdata = arguments[0];
		VCOM.set('pdata',pdata);
		if (pdata && pdata.lists && pdata.lists.length && pdata.lists[0].current) {
			VCOM.set('activelist',pdata.lists[0]);				
		} else {
			VCOM.set('activelist',null);
		}
		
		$('body').trigger($.Event('playerdata.update',{pdata:pdata}));		
		
	}	
	
	var $window = $(window);
	
	Challenge.find = function(element) {		
		for (element = $(element);element.length;element = element.parent()) {
			var ret = element.data('challenge');
			if (ret) return ret;			
		}
	}
	
	VCOM.on('login',function(){
		if (!VCOM.get('valid')) storage.reset();
		var $challenge = $('#challenge');
		var challenge = new Challenge($challenge);
		$challenge.data('challenge',challenge);		
	});

	VCOM.on('change:mode',function(vcom,mode) {
		var $challenge = $('#challenge');
		var challenge = $challenge.data('challenge');
		
		if (challenge) {
			if (mode == 'play') {
				if (!$challenge.data('started')) {
					$challenge.data('started',true);
					challenge.start();					
				}
				challenge.element.focus();				
			} else {
				$challenge.find(':focus').blur();
			}
		}
	});
	
				
	function Challenge(element) {		
		this.element = element = $('<div/>').data('challenge',this).attr('tabindex',0)
		                         .addClass('vcom-challenge resizable')
		                         .empty().appendTo(element);
		
		VCOM.challenge = this; 
		
		var _this = this;		
		var secret = null;		
		var questionPane = $('<div class="questionPane"></div>').appendTo(element)
		                   .css({'transition':'all 0s ease'});
		var nextButton = $('<button class="next" ><span>&#8250;</span></button>');
		var nextArea = $('<div/>').css({top:0,bottom:0,right:0,position:'absolute'})
						.append(nextButton).appendTo(element);
		
				
		element.on('click','.next,.qnext',next);
		
		var shownAt = 0;
				
		// setup event handlers
		element
		  .on('challenge.questionComplete',function() {
				// on complete, deactivate slide and activate next button	
			  var question=questionPane.find('.challenge-slide.active')
				            .removeClass('active')
				            .addClass('complete')
				            .attr('adata',null);
			  var adata =question.data('adata');
			  if (adata && adata.word) question.attr('word',adata.word);
			  nextButton.addClass('active');						  
		  })
		  .on('challenge.go',function(e) {			
			  $(':focus').blur();			 
		  })
		  .on('challenge.update',function(data) {
			  if (data.pdata)  {
				  VCOM.playerData(data.pdata);
			  }
			  if (data.secret) {
				  secret = data.secret;
			  }
		  });
		
		storage.init();
		element.on('challenge.reset',storage.reset)
		       .on('challenge.update challenge.go',function() {
			element.doTimeout('storestate',1000,function(){
				window.localStorage.setItem('challenge.frames', questionPane.html() );
				window.localStorage.setItem('challenge.secret', secret );
			});
		});
						
		this.playerData = function() {
			return pdata;
		}
		
		this.start = function() {
			element.focus();
			var startFrame = 0;
			if (window.localStorage) {
				try {
					secret = localStorage.getItem('challenge.secret');		
				} catch (e) {
					secret = null;
				}
			}		
			resized();
			
			$.ajaxq('challenge',{
				url:VCOM.server+'/start.json',
				type:'POST',
				dataType:'json',				
				data:{secret:secret},
				success:function(data) {			
					try {
						if (data.secret) secret = data.secret;
						if (data.cmd == 'resume') {							
							frame( resume(), false);
							trigger('resume');	
							trigger('update',data);	
						} else {
							questionPane.empty();
							activate(data);						
							frame(0, false);
						}	
					} catch (e) {
						VCOM.error({cause:e,message:'Error starting the challenge'});
					}
				},
				error: VCOM.errorHandler(function(){ _this.start(); })					
			});
		};
		
		function resume() {
			var frms = localStorage.getItem('challenge.frames');			
			if (frms) {				
				var startFrame =0;
				questionPane.empty().append(frms);
				
				var frames = questionPane.children('.challenge-slide');						
				
				for (var i=0;i<frames.length;i++) {
					var f = $(frames[i]);
					if (f.attr('adata')) {
						f.data('adata', dcd( f.attr('adata') ));
					}
					var qtype = Challenge.types[ f.data('slide-type') ] || Challenge.types[ 'interstitial' ];
					if (qtype) qtype(f);
					
					if (f.hasClass('active')) {
						f.trigger($.Event('challenge.nextQuestion',{challenge:_this,slide:f}));
						shownAt = new Date().getTime();
					}
					if (f.hasClass('selected')) {
						startFrame = i;
						f.removeClass('selected');
					}
				}						
				VCOM.parse(questionPane);
				layoutSlides();
				return startFrame;
			}
			
		}
		
		function activate(data) {
			if (data.cmd == 'newround') trigger('newround');
			var slide = $(decodeBase64(data.code))
						.data('adata', dcd(data.adata))
						.attr('adata',data.adata)
						.addClass('active')
						.appendTo(questionPane);	
			slide.appendTo(questionPane);
			
			layoutSlides();
			delete data.code;
			var qtype = Challenge.types[ slide.data('slide-type') ] || Challenge.types[ 'interstitial' ];
			if (qtype) {
			    qtype(slide);
				slide.trigger($.Event('challenge.nextQuestion',{challenge:_this,slide:slide}));
				shownAt = new Date().getTime();
			}
			VCOM.parse(slide);
			trigger('update',data);
		}
		
		this.getSecret = function() { return secret; }
		this.setSecret = function(s) {  secret=s; }
		
			
		this.saveAnswer = saveAnswer;
		function saveAnswer( questionSlide, data ) {
			data = data || {};
			data.t= (new Date()).getTime();
			data.rt = data.t - shownAt;
			data.secret = secret;
			$.ajaxq('challenge',{
				url: VCOM.server+'/saveanswer.json',
				dataType:'json',
				type: 'POST',
				data: data,
				success:function(responseData) {					
					try {
						questionSlide.addClass('saved').trigger($.Event('challenge.save', responseData ));					
						trigger( 'update', responseData );				
						//if (window._gaq) _gaq.push(['_trackEvent', 'challenge', 'answer' ]);
					} catch (e) {
						VCOM.error({cause:e,message:'We had a problem saving your answer'});
					}					
				},
				error:  VCOM.errorHandler( function() { saveAnswer(questionSlide, data); } )
			});
			return _this;
		}
			
		function trigger(type, data) {			
			element.trigger($.Event('challenge.'+type,data || {}));
		}
		
		var showingNext = false;
				
			
		this.next = next;
		this.prev = prev;
		
		function next() { 
			var e = $.Event('challenge.next');
			questionPane.find('.challenge-slide.selected').trigger(e);
			var ret =  (e.isDefaultPrevented()) ? false : go(1);
			if (ret) VCOM.history.push(prev);
			return ret;
		}
		
		function prev() { 
			var e = $.Event('challenge.prev');
			questionPane.find('.challenge-slide.selected').trigger(e);
			var ret = (e.isDefaultPrevented()) ? false : go(-1);	
			if (ret) VCOM.history.push(next);
			return ret;
		}
		   
		
		
		this.go = go;
		function go( offset ) {
			
			if (questionPane.data('animated')) return false;
			
			var fn = frame();
			var slide = $(questionPane.children('.challenge-slide')[fn]);			
			
			if (slide.hasClass('complete') && slide.hasClass('wide')) {
				if (offset == 1 && !slide.hasClass('bside')) {
					slide.addClass('bside');
					if (element.hasClass('small')) {
						snap(fn,true);
						trigger('go',{slide:slide,bside:true});
						return true;
					}
				} else if (offset == -1 && slide.hasClass('bside')) {
					slide.removeClass('bside');
					if (element.hasClass('small')) {
						snap(fn,true);
						trigger('go',{slide:slide,bside:true});
						return true;
					}
				}
			}			
			
			return (offset && frame(fn+offset,true));
		}
		
		this.slide = function() {
			return questionPane.find('.challenge-slide.selected');
		}
		
		this.frame = frame;

		function frame( idx, animated ) {
			var frames = questionPane.children('.challenge-slide');			
			if (!arguments.length) {
				// TODO: there is a better way to do this
				return frames.index( questionPane.find('.challenge-slide.selected') );
			}
			if (idx == frames.length && $(frames[idx-1]).hasClass('complete') && !showingNext) {
				showingNext = true;
				nextButton.removeClass('active');									
				$.ajaxq('challenge',{
					url:VCOM.server+'/nextquestion.json',
					type:'POST',
					dataType:'json',
					data:{secret:secret},
					success:function(content) {							
						try {
							secret = content.secret;
							if (frames[idx-1]) {
								$(frames[idx-1]).trigger('challenge.hideFrame');
							}
							activate(content);
							frame(idx,animated);
						} catch (e) {
							VCOM.error({cause:e,message:"Sorry, there has been an error rendering the next question."});
						}
					},
					error: VCOM.errorHandler( function() { frame(idx,animated); } ),	
					complete:function(){
						showingNext = false;
					}
				});
				return true;				
			}
			var newFrame = Math.max(0,Math.min(frames.length-1,idx));			
			var newSlide = $(frames[newFrame]);
			
			if (!newSlide.hasClass('selected')) {
		    	snap(newFrame,animated);		    	
		    	if (newSlide.hasClass('complete')) {
		    		nextButton.addClass('active');
		    	} else {
		    		nextButton.removeClass('active');
		    	}		    	
		    	newSlide.trigger($.Event('challenge.showFrame',{frame:newFrame,slide:newSlide}));
		    	trigger('go',{slide:newSlide});
		    	return true;
			}
			
			return false;
		}
		
		element.on('challenge.newround',function() {
			questionPane.empty().css('transform','translate3d(100%,0,0)');
			VCOM.history.remove(_this.next,true);
			VCOM.history.remove(_this.prev,true);
			
		});
		
		questionPane.on('dragstart selectstart',function(evt){
			evt.preventDefault();
		});
		
		var excludes = {'INPUT':true,'SELECT':true,'TEXTAREA':true,'RADIO':true,'CHECKBOX':true}; 
		element.on('keydown',function (e) {
			if (e.target.nodeName in excludes) return;
		    if (e.which == 39 || e.which == 13 || e.which == 32) {
		    	e.preventDefault();
		    	_this.next();		    	
		    } else if (e.which == 37) {
		    	e.preventDefault();
		    	_this.prev();
		    }
		    var ch = String.fromCharCode(e.which);
		    if (ch.length) {
			    questionPane.find('.challenge-slide.selected.active a[accesskey~="'+ch+'"]')
			      .trigger('click');
		    }
		});
		
		function layoutSlides() {
			var useWide = element.hasClass('small');
			var x=0;
			questionPane.children('.challenge-slide').each(function(){
				var $this=$(this)
				$this.css('transform','translate3d('+x+'%,0,0)').data('offset',x);
				x += (useWide &&  $this.hasClass('wide')) ? 200 : 100;
			});
		}
		
		function isBSide( frame ) {
			return (element.hasClass('small') && frame.hasClass('bside'));
		}
		
		function snap(newFrame,animated) {		
			var frames = questionPane.children('.challenge-slide');
			var $frame = $(frames[newFrame]);
			var offset = $frame.data('offset');
			if (isBSide($frame)) offset+=100;
			
			if (!$frame.hasClass('selected')) {
				$frame.addClass('selected').siblings('.selected').removeClass('selected');			
			}
			var cx = getPosition(questionPane);
			if (cx != offset) {
				questionPane.css({
					'transition-duration': animated ? '.3s' : '0s',
		    		'transform': 'translate3d(-'+offset+'%,0,0)'	
		    	});
			}
		}
		window.snap = snap;
		
		element.on('touchstart',function(evt) {
			if (evt.originalEvent.touches) {
				if (evt.originalEvent.touches.length > 1) return;
				evt = evt.originalEvent.touches[0];
			}
			var dragging = false;
			var startFrame = _this.frame();
			var frames = questionPane.children('.challenge-slide');	
			var $frame=$(frames[startFrame]);
			var start = { x: evt.pageX, y: evt.pageY };
			var last =  { x: start.x,   y: start.y };

			// fix transform where it is now
			questionPane.css({'transition-duration':'0s',
				              'transform':'translate3d('+getPosition(questionPane).x+'px,0,0)'});
			
			$window.on('touchmove',onDrag).one('touchend',end);
			
			function onDrag(evt) {				
				if (evt.originalEvent.touches && evt.originalEvent.touches.length > 1) {
					cancel();
					return;
				}
				var touchPoint = evt.originalEvent.touches ? evt.originalEvent.touches[0] : evt;
				var dx = touchPoint.pageX-last.x, 
				    dy = touchPoint.pageY-last.y;
				last.x = touchPoint.pageX;
				last.y = touchPoint.pageY;
				
				if (dragging || Math.abs(dx) > Math.abs(dy)) {
					dragging = true;	
					questionPane.css({
						'transition-duration':'0s',
						'transform':'translate3d('+(dx+getPosition(questionPane).x)+'px,0,0)'						
					});
					evt.preventDefault();
					evt.stopPropagation();
				} else {
					cancel();
				}
			}
			function cancel() {
				$window.off('touchmove',onDrag).off('touchend',end);
				if (dragging) {
					dragging = false;
				}
				snap(frame(),true);
			}
			function end(evt) {
				$window.off('touchmove',onDrag);
				if (dragging) {
					dragging = false;
					var touchPoint = evt.originalEvent.changedTouches ? evt.originalEvent.changedTouches[0] : evt;				
					var dx = touchPoint.pageX-start.x, dy = touchPoint.pageY-start.y;
					if (Math.abs(dx) > 30) {
						if (! ((dx>0) ? prev() : next()) ) {
							snap(frame(),true);						
						}
					} else {
						snap(frame(),true);						
					}
				} else {
					snap(frame(),true);	
				}
			}	
		});	
		
		element.on('resized',resized);
		
		function resized() {
			var w = element.width();
			if (w > 0) {
				if (w <= 760 && !element.hasClass('small')) {
					element.removeClass('large').addClass('small');
					layoutSlides();
					snap(frame(),true);	
				} else if (w > 760 && !element.hasClass('large')) {
					element.removeClass('small').addClass('large');
					layoutSlides();
					snap(frame(),true);					
				}
				nextArea.css('width',(Math.max((w-960)/2,0)+50)+'px');
			}
		}
	
	Challenge.types = { 
		"choice": multipleChoiceQuestion,
		"spelling": spellingQuestion,
		"interstitial" : interstitial
	};
	
	function interstitial(pane) {		
		if (pane.hasClass('active')) {
			window.setTimeout(function(){
				pane.trigger('challenge.questionComplete');					
			},parseInt(pane.data('delay-next')) || 750 );			
		}
	}	
	
   	function multipleChoiceQuestion(question) {
   		if (!question.hasClass('active')) return;
   		
   		var choices = question.find('.choices a').not('.correct, .incorrect');
   		var tries = 4-choices.length;   	
   		var lastPickTime = 0;
   		choices.one('click',function () {
			lastPickTime = +(new Date());   
			var choice = $(this).off('click');
			var evt = {
				correct: (choice.attr('nonce')== question.data('adata').nonces[0]),
				tries: ++tries
			};			
			choice.trigger($.Event('challenge.pick',evt));
			
			if (evt.correct) {
				choices.off('click');
				choice.addClass('correct');		
				question.trigger('challenge.questionComplete');	
			} else {
				choice.addClass('incorrect');
				if (evt.tries===1) question.find('.pointValue').html('0');
			}
			question.toggleClass('incorrect',!evt.correct);
			var messages= (evt.correct) ? correctMessages[evt.tries-1] : wrongMessages[evt.tries-1];
			if (evt.tries===1) {
				Challenge.find(question).saveAnswer(question,{ a: choice.attr('nonce') });
			}
			VCOM.status.message($('<h3>'+messages[Math.floor(Math.random()*messages.length)]+'</h3>'),evt.correct);			   
		});
   		
		// lifelines
   		(function(){
   			var lifelines = question.find('.lifeLines');
			var content = question.find('.lifeLineContent');
			if (tries == 0) {
				question.one('challenge.pick',function() {
					question.doTimeout('showlifelines'); // cancel lifelines
					lifelines.slideUp().find('a').off('click');
					content.slideUp();
				});
				question.doTimeout('showlifelines',7000,function(){
					lifelines.fadeIn().find('a').one('click',function() {
						lifelines.slideUp().find('a').off('click');
						var type = $(this).data('type');				
						var pv=question.find('.pointValue');
						pv.text( parseInt(pv.text()) / 2 );
						$.ajaxq('challenge',{
							url:VCOM.server+'/hint.json',
							type:'POST',
							dataType:'json',
							data:{ secret: secret, type: type },
							success: function(result) {
								try {
									trigger('update',result);
									switch (type) {
										case 'E':								
											content.append($('<vcom:examples count="1" />').attr('word',result.word));
											VCOM.parse(content);
											content.fadeIn();								
											break;
										case 'F':								
											result.nonces.forEach(function(n) { 
												$('.choices a[nonce='+n+']').addClass('incorrect').off('click');
											});
											break;
										case 'D':								
											 content.append(result.def);
										default:
											content.fadeIn();						
									}
								} catch (e) {	
									VCOM.error({cause:e,message:'There was an error rendering the lifelines'});
								}							
							},
							error: VCOM.errorHandler()
						});				
					});			
					
				});
			}
   		})();
				   		
		var wrongMessages = [
      	  ["Nope, that wasn\'t it, take another look.",
      	   "Strike 1! Give it another try!",
      	   "Um, no. What's your second guess?"],
      	  ["That wasn\'t it either",
      	   "Look at the bright side, now you have a 50/50 chance!",
      	   "You're getting warmer...",
      	   "Survey says: you're wrong again."],
      	  ["Well, it should be pretty easy now, take a wild guess.","You can't miss now","Easy enough for you now?",
      	    "I guess today isn't your lucky day.","See if you can spot the right answer now.",
      	    "Let me make this real easy for you."]	  
      	];
      	
      	var correctMessages = [
     	  ["Nice Job!","Correct!"],
     	  ["You got it!","Bingo!","We knew you'd figure it out!","We have a winner!"],
     	  ["Third time's a charm, huh?"],  
     	  ["That's the one!","You're a genius!"]	       	  
     	];		
   	}
   	
   	function spellingQuestion(question) {
   		if (!question.hasClass('active')) return;
   		
   		var spellingField = $('input.wordspelling', question).on('keypress', function(e) {
			if (e.which == 13) checkSpelling();		    
		});
   		
   		/*question.one('challenge.showFrame',function() {   		
			question.doTimeout('focus',500,function(){
   				//if (question.hasClass('active selected')) {
   				//	VCOM.playAudio(question);
   				//	spellingField.focus();   					
   				//}
   			});   			
   		});*/
   				
   		$('.playword',question).on('click',function(){ 
   			 VCOM.playAudio(question); 
   			 spellingField.focus() 
   		});
   		
		var spellitButton = $('button.spellit', question).on('click',checkSpelling);
		
		var surrenderButton = $('button.surrender', question).on('click', function() {
			showAnswer();
			VCOM.status.message( '<h3>The correct spelling is <span class="correctspelling">' + 
                       question.data('adata').word +
                       '</span>.</h3>' );
		});
		
		var sentenceBlanked = $('.questionContent .blanked', question);	   	
		var sentenceComplete = $('.questionContent .complete', question);	   	
		var messagePane = $('.messagePane', question);
				
		function checkSpelling() {	
			
			var guess =spellingField.val().trim().toLowerCase();
			if (guess.length == 0) {
				$('.playword',question).trigger('click');
				return;
			}
			var data = question.data('adata');
			
			var evt = {
				correct: (data && data.acceptedAnswers.indexOf(guess)!=-1),
				tries: parseInt(question.attr('guesses')||0)+1
			};						
			question.attr('guesses',''+evt.tries);
			question.trigger($.Event('challenge.pick',evt));
			if (evt.correct) {
				showAnswer();
			} else {
				if (evt.tries > 2) question.addClass('cansurrender');			
			}
			
			spellingField.blur();
			setTimeout(function(){ spellingField.trigger('focus') },2000);
			
			var tries = Math.min(3, evt.tries);
			var messages=(evt.correct)?correctMessages[tries-1]:wrongMessages[tries-1];
			var randmsg = (evt.correct || tries < 3) ? messages[Math.floor(Math.random()*messages.length)] :
				                                       messages[(evt.tries-1) % messages.length];
			Challenge.find(question).saveAnswer(question,{a:guess});
			VCOM.status.message( '<h3>'+randmsg+'</h3>',evt.correct );			   
		}
								
		function showAnswer() {
			answered = true;
			spellingField.attr('disabled', true);
			spellingField.blur();
			spellingField.off('keypress focus')
			surrenderButton.off('click');
			spellitButton.off('click');
			
			question.trigger('challenge.questionComplete');
		}
				
		var wrongMessages = [
    	  ["Nope, that wasn\'t the answer, try again.","Strike 1! Give it another try!","Um, no. What's your second guess?"],
    	  ["That wasn\'t it either","Survey says: you're wrong again."],
    	  ["That's not right.", "Wrong."]
    	];

        var correctMessages = [
       	  ["Nice Job!","Correct!"],
       	  ["You got it!","Bingo!","We knew you'd figure it out!","We have a winner!"],
       	  ["Third time's a charm, huh?"]
       	];
   	}
   	
   	element.one('challenge.resume',function(){
	   	element.find('.vcom_questionRater, form.questionRater').remove();	
	   	element.find('.question.rating_inprog').show().removeClass('.rating_inprog');
   	});   	
   	element.on('challenge.nextQuestion',function(evt){
    	element.find('.challenge-slide.complete .vcom_questionRater').remove();
   		var slide = evt.slide;
   		if (slide && slide.hasClass('active')) {
   			var question = slide.find('.question');
   			var rater = $('<a class="vcom_questionRater" href="javascript:void(0)">Improve this question</a>')
   				.appendTo(question)
   			    .on('click',function(){
   			    	question.addClass('rating_inprog');
		   			var form = $('<form class="question questionRater standard"></form>')
			            .append('<h1>Question Feedback:</h1>')
			            .append('<div class="field"><label>Select a category:</label><select name="reason" class="reason" required>\
									<option value="">Select...</option>\
									<option value="typo">Typo / Misspelling</option>\
									<option value="grammar">Grammatical error</option>\
									<option value="no correct answer">No correct answer</option>\
									<option value="more than one answer">More than one correct answer</option>\
									<option value="innaproprate">Inappropriate material</option>\
									<option value="obvious">Answer is obvious</option>\
									<option value="other">Other</option>\
								    </select></div>')
						.append('<div class="field"><label>Your Comments:</label><textarea name="comments" maxlength="500"></textarea></div>')
						.append('<div class="field"><button class="standard gray cancel" type="reset">Cancel</button>\
								 <button class="submit standard green" type="submit">Submit your feedback</button></div>')
						.css({position:'absolute',display:'none'})
						.prependTo(question.parent())
						.on('submit',function(e){
							e.preventDefault();
							rater.addClass('complete').html('Thanks for the feedback!');							
							hide();
							var params = {secret:secret};
							$.each(form.serializeArray(),function(){ if (this.name) params[this.name]=this.value; });
							$.ajaxq('challenge',{
								url:VCOM.server+'/ratequestion.json',
								type: 'POST',
								dataType: 'json',
								data:params,
								success:function(data){							
									trigger('update',data);
								},
								error: VCOM.errorHandler()								
							});	
						})
						.find('button.cancel').on('click',function(e){
							hide();							
						}).end().fadeIn(function(){
		   					$(this).css({position:'relative'})
		   						.doTimeout('focus',10,'focus');  					
		   				});
		   			
	   				question.fadeOut();
	   					   				
	   				function hide() {
	   			    	question.removeClass('rating_inprog');
	   					question.fadeIn();
	   					form.css({position:'absolute'}).fadeOut(function(){
	   						form.remove();					
	   						trigger('update');
	   					});
	   				}
	   				
	   				slide.one('challenge.hideFrame',hide);
		   			
   			});
   		} 
   	});
   	
	   	
   	   	
	} /* end challenge */
   			
	var posMap = {'n':'noun','v':'verb','a':'adjective','r':'adverb'};
	

	function getPosition(el) {
		var matrix = el.css('-webkit-transform');
		matrix =matrix.split(')')[0].split(', ');
		return {x: +(matrix[12] || matrix[4]),
			    y: +(matrix[13] || matrix[5]) };
	}
	
			
	function makeMessage(answer) {	
		var msg = '<div class="def"><strong>'+capitalize(answer.word)+'</strong> ';
		msg+='is ';
		msg+=(answer.pos == 'a' || answer.pos == 'r')?'an ':'a ';
		msg+=posMap[answer.pos]+' that means <i>';
		if (answer.pos=='v') msg+='to ';
		msg+=answer.definition+'</i>.</div>';
		return msg;
	}
	
	function capitalize(s) {
		return s.substring(0,1).toUpperCase()+s.substring(1);
	}
   	
	function r13(s) { return s.replace(/[a-zA-Z]/g,function(c){return String.fromCharCode((c<="Z"?90:122)>=(c=c.charCodeAt(0)+13)?c:c-26);}) };
	function decodeBase64(s) {
	    var e={},i,b=0,c,x,l=0,a,r='',w=String.fromCharCode,L=s.length;
	    var A="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	    for(i=0;i<64;i++){e[A.charAt(i)]=i;}
	    for(x=0;x<L;x++){
	        c=e[s.charAt(x)];b=(b<<6)+c;l+=6;
	        while(l>=8){((a=(b>>>(l-=8))&0xff)||(x<(L-2)))&&(r+=w(a));}
	    }
	    // THIS LITTLE GEM OF A HACK CONVERTS A JAVA UTF-8 ENCODED STRING TO A JAVASCRIPT STRING
	    // SEE: http://ecmanaut.blogspot.ca/2006/07/encoding-decoding-utf8-in-javascript.html
	    return decodeURIComponent(escape(r));
	};
	function dcd(s) { return (s == null || !s.length) ? undefined : $.parseJSON(r13(decodeBase64(s))); }
	function dct(s) { return (s == null || !s.length) ? undefined : r13(decodeBase64(s)); }
	
	function shuffle(o){ 
	    for(var j, x, i = o.length; i; j = parseInt(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
	    return o;
	}	
});