Module.define('vcom/examplebrowser', ['jquery','date'],
function( $ ) {
	var domains = [ null,'F','A','N','B','S','M','T'];
	var domainNames = ['All Sources','Fiction','Arts / Culture','News','Business','Sports','Science / Medicine','Technology'];
	
	this.construct = function() {
		$this = $(this);
		var ret = $this.data('examplebrowser');
		return ret || (ret = new ExampleBrowser(this));
	};
	
	function ExampleBrowser(element,options) {
		var _this = this;
		this.clear = clear;
		this.next=next;
		this.prev=prev;	
		
		options=options||{};	
		if (!options.displayWindow) options.displayWindow=3;
		if (element) {
			element = $(element).empty().addClass('vcom_examples').data('examplebrowser',this);
			if (element.attr("count")) {
				options.displayWindow=parseInt(element.attr("count"));
			}
			if (element.attr("filter")!=null) {
				options.filter=parseInt(element.attr("filter"));
			}
			if (element.attr("domains")=='false') {
				options.hideDomains=true;
			}
		}
		
		this.options = options;
		
		var root = $('<div class="exampleBrowser"></div>');
		this.element=root[0];
						
		var resultsContainer = $('<div class="container"></div>').appendTo(root);				
		var resultsList	= $('<div class="results"><div class="messages">loading examples...</div></div>').appendTo(resultsContainer);

		$('<div class="buttons"><a class="prev ss-navigateleft" title="prev">Prev</a><a class="next ss-navigateright right" title="next">Next</a></div>')
		   .appendTo(root)
		   .find('a.next').on('click',next).end()
		   .find('a.prev').on('click',prev).end();
		
		if (!options.hideDomains) {
			var domainDiv = $('<div class="domains ss-navigatedown"></div>');			
			for (var i=0;i<domains.length;i++) {
				var item = $('<a href="javascript:void(0)"></a>')
				     .text(domainNames[i]).attr('title',domainNames[i])
				     .data('code',domains[i])
				     .appendTo(domainDiv);				
				if (i==0) item.addClass('selected');
			}
			domainDiv
				.appendTo(root)
				.find('a')
				.on('click',function(){
					domainDiv.toggleClass('open');
					$this=$(this);
					if (!$this.hasClass('selected')) {
						$this.addClass('selected').siblings().removeClass('selected');
						var code = $this.data('code');
						if (code) options.domain = code;
						else delete options.domain;				
						if (params) {	
							params.startOffset =0;
							data.reset();
							refresh();						
						}
					}
				});
		}
		
		
		if (element) {
			$(element).empty().append(root);
		}
		
		var params = null;
		
		var searchTerm = null; 
		this.search = function(words) {
			if (words != searchTerm) {
				searchTerm=words.trim();
				params={'query':words,'maxResults': data.displayWindow * 6,'startOffset':0 };				
				data.reset();
				refresh();	
			}
			return false;			
		};
		function refresh() {			
			if (params.query) {
				params.domain = options.domain;		
				if (options.filter != undefined) params.filter=options.filter;
				loading=true;				
				postJSONXS(params,function(r) { 
					loading=false;
					data.totalResults = r.totalHits;						
					data.sentences.push.apply(data.sentences,r.sentences);
					renderData(true);
				});			
			}
		};
		
		function clear() {
			resultsContainer.empty();
			data.reset();
			element.trigger('exampleBrowser.clear');
		}
		
			
		var data = {
			'displayPtr': 0,
			'displayWindow': options.displayWindow,
			'sentences'	:[],
			'totalResults' : 0,
			'renderHTML' : renderHTML,
			'hasNext' : function() { return this.displayPtr+this.displayWindow < this.totalResults; },
			'hasPrev' : function() { return this.displayPtr>0; },
			'reset' : function() { this.displayPtr=0; this.sentences=[]; this.totalResults=0; }
		};
		
		var loading = false;
		
		
		if (element && element.attr("word")) {
			this.search(element.attr("word"));
		}		
				
		function renderData( fromRight ) {
			if (data.sentences.length==0) {
				slide($('<div class="results"><div class="messages">no examples found</div></div>'),fromRight);
			} else {
				slide($('<div class="results"></div>').append(template(data)),fromRight);
			}
			root.toggleClass('hasPrev',data.hasPrev());
			root.toggleClass('hasNext',data.hasNext());
			root.trigger('exampleBrowser.load',params.query);
		}
		
		function slide(offscreen, rtl ) {	
			var to = (resultsContainer.width() + 10);
			resultsContainer.height(resultsContainer.height());
			root.addClass('animating');
			offscreen.css('left',( rtl ? to : -to) +"px")
				.appendTo(resultsContainer)
				.animate({left:0},500);			
			resultsContainer.animate({height:offscreen.height()},500);
			var toRemove = resultsList;			
			resultsList.animate({left: rtl ? -to : to},500,function(){
				toRemove.remove();
				resultsContainer.css('height','auto');
				root.removeClass('animating');				
			});
			resultsList = offscreen;
		}
				
		function next() {
			if (data.hasNext()) {
				if (data.displayPtr + (data.displayWindow*3) >= data.sentences.length) {
					// need to load more data!
					if (!loading) {
						loading=true;
						params.startOffset = data.sentences.length;
						postJSONXS(params,function(r) { 
							loading=false;							
							data.totalResults = r.totalHits;						
							data.sentences.push.apply(data.sentences,r.sentences);
						});	
					}
				}
				if (data.displayPtr < data.sentences.length+data.displayWindow) {
					data.displayPtr += data.displayWindow;
					renderData(true);
				}
			}					
		}
		function prev() {
			if (data.hasPrev()) {
				data.displayPtr -= data.displayWindow;
				renderData(false);
			}						
		}		
	}	
		
	function renderHTML(result) {	
		var sent = $('<div class="sentence"></div>');
		var str = result.sentence; 
		var offsets=result.offsets;
		var last = 0;
		for (var i=0;i<offsets.length;i+=2) {
			sent.append(document.createTextNode(str.substring(last,offsets[i])));					
			sent.append($("<strong></strong").text(str.substring(offsets[i],offsets[i+1])));
			last = offsets[i+1];								
		}
		sent.append(document.createTextNode(str.substring(last)));
		return $('<li></li>').append(sent);
	}

	function template(data) {
		var ul = $('<ul></ul>');
		for (var s=data.displayPtr;s<Math.min(data.sentences.length,data.displayPtr+data.displayWindow);s++) {
			var result = data.sentences[s];	
			var title = (result.corpusId=='GUT') ? result.volume.author : result.volume.corpus.name;
			var pubDate = (result.volume.datePublished) ? new Date(result.volume.datePublished).format('mediumDate') : ''
			var href = 	(result.corpusId=='GUT') ? 	'http://www.gutenberg.org/ebooks/'+result.volume.locator : result.volume.locator;
		
			var li = renderHTML(result);
			var sourceLink = $('<a target="_blank" class="source"></a>').attr('href',href).appendTo(li);		
			$('<span class="corpus"></span>').text(title).appendTo(sourceLink);
			if (result.corpusId!='GUT') $('<span class="date"></span>').text(pubDate).appendTo(sourceLink);
			li.appendTo(ul);
		}
		return ul;		
	}
	
	function postJSONXS(params,onload) {		
		if (!params) params = {};
		$.ajax({
			url: 'http://corpus.vocabulary.com/api/1.0/examples.json',
			type:'GET',
			data: params,
			dataType:'json',
			success: function(result){ onload(result.result); }			
		})
	}
	function encodeParams(params) {
		var ret = [];
		if (params) {
			for (var p in params) {
				var value = params[p];
				if (value == undefined || value == null) {
					;
				} else if (typeof value == 'string' || typeof value == 'number' || typeof value == 'boolean') {				
					ret.push(p+'='+encodeURIComponent(value));
				} else if (value.length != undefined) {
					for (var i=0;i<value.length;i++) {
						ret.push(p+'='+encodeURIComponent(value[i]));
					}
				} else {
					throw new Error("cannot serialize parameter: "+p+" of type "+(typeof value));
				}
			}
		}
		return ret.join('&');
	}
	
	this.ExampleBrowser = ExampleBrowser;
	
});
