Module.define('vcom/listtab',['jquery','vcom/base','format','vcom/status'],function($,VCOM,Format){
	var userVocabOptions = {
		'learning':	{system:true,url:VCOM.server+'/words.json',title:'Words I\'m Learning','class':'system learning',
			 description:'A list of the words you are currently learning that you have not yet mastered.',
			 empty:'You\'re not currently learning any words. <a href="/">Keep playing</a> and we\'ll add some for you, or <a href="/dictionary/">find some words to add.</a>',
			 params:{learning:true,order:'progress',asc:true,maxResults:20}},
		'mastered': {system:true,url:VCOM.server+'/words.json',title:'Words I\'ve Mastered', 'class':'system mastered',
			 description:'A list of all the words you have mastered. Nice Job.',
			 empty:'Unfortunately, you haven\'t mastered any words yet. But don\'t despair, <a href="/">keep playing</a> and we\'re sure you\'ll be a <a href="/leaderboards/today/L09">Logophile</a> in no time!',
			 params:{mastered:true,order:'mastered',asc:false,maxResults:20}},
		'all': {system:true,url:VCOM.server+'/words.json',title:'Words I\'ve Played', 'class':'system allwords',
			empty:'You haven\'t answered any questions yet. <br><br><a href="/">Play Vocabulary.com</a> and words you play will appear here.',
			description:'All the words you have ever played on Vocabulary.com',params:{order:'word',maxResults:20}}
	};
	
	var loaded = false,
		statusBarLearn,	statusBarProgress,
		currentList = undefined,
		$progress;
	
	function configure() {
		currentList = undefined;
		loaded = false;
		statusBarLearn = VCOM.status.create('<div>Start learning this list</div>').addClass('lists startlearn hidden'); 
		statusBarLearn.on('click','header',function(){
			var listId = $('#lists').find('.wordlist').data('list-id');
			if (listId) VCOM.startList(listId,function() {
				statusBarProgress.addClass('open');
			});   
		});
		
		$progress = $('<div><i class="progress"/>\
				<span class="label">Your progress on:</span><span class="title"></span></div>');
		var $options = $('<section><button class="standard green play">play now</button><button class="standard red stop">stop learning</button></section>');
		
		statusBarProgress = VCOM.status.create($progress)
							 .addClass('lists progress hidden')
							 .append($options)
							 .on('click','header',function(){
								statusBarProgress.toggleClass('open');
							 })
							 .on('click','button.stop',function(){
								statusBarProgress.removeClass('open');
								var data = $('#lists').find('.wordlist').data();
								if (data.listId) VCOM.stopList(data.listId);   
							 })
							 .on('click','button.play',function(){
								statusBarProgress.removeClass('open');
								var data = $('#lists').find('.wordlist').data();
								if (data.listId) VCOM.startList(data.listId,function() {
									VCOM.mode('play');
								});  
							 });
		
		$('nav .listsTab').on('click',function(){
			if (VCOM.mode()=='lists') VCOM.showList();
		});
		
		var scrolling = false;
		$('#lists')
			.on('scroll',function(evt) {
				if (this.scrollTop > 0 && 
					this.scrollHeight - (this.scrollTop + this.clientHeight) < 500  ) {					
					$(this).find('.hasmore').trigger('fetch');
				}
			})
			.on('input','.searchlists input',function(){
				$(this).doTimeout('search',200,searchLists,$(this).val());
			});
	}
	
	function searchLists(query) {
		query = (query || "").trim();
		if (query.length > 2) {
			$.post(VCOM.server+'/list/search.json',{q:'+(name:'+query+'*) (featuredate:[0 TO '+Date.now()+'])'},function( data ){
				var $results = $('#lists .search-results ul').empty();
				if (data.results) {
					$results.append(data.results.map(function(item){
					var entry = $('<li role="button" ><a ></a><div class="description"></div></li>');
					entry.find('a').text(item.name).attr('href','/lists/'+item.wordlistid);
					entry.find('.description').text(Format.truncateNicely(Format.stripHTML(item.description),100));
					entry.attr('listid',item.wordlistid);
					return entry;
				}));
				}
			});
		} else {
			$('#lists .search-results ul').empty();
		}
	}
	
	
	VCOM.on({
		'login': configure,
		'change:mode' : function(vcom,mode) {
			if (mode == 'lists') {		
				if (!loaded) {
					VCOM.showList();			
				} else {
					updateListProgress();
				}
			} else {
				$('.statusbar.lists').hideStatus();
			}
		}		
	});
	
	VCOM.startList = function(id,callback) {
		$.post(VCOM.server+'/list/start.json',id==null?null:{id:id},function(data) {
			if (data && data.pdata) {
				VCOM.playerData(data.pdata);			
				updateListProgress();
			}
		    if (typeof callback ==='function') callback.call(VCOM);
		},'json').fail(VCOM.errorHandler(function(){
			VCOM.startList(id,callback);
		}));
	}
	
	VCOM.stopList = function(id,callback) {
		$.post(VCOM.server+'/list/stop.json',{id:id},function(data) {
			if (data && data.pdata) {
				VCOM.playerData(data.pdata);			
				updateListProgress();
			}
			if (typeof callback ==='function') callback.call(VCOM);
		},'json').fail(VCOM.errorHandler(function(){
			VCOM.stopList(id,callback);
		}));
	}
	
	VCOM.showList = function(id,callback) {
		if (VCOM.mode()=='lists' && loaded) {
			var last = currentList;
			VCOM.history.push(function(){ VCOM.showList(last); });
		}
		
		loaded=true;
		VCOM.mode('lists');
		
		var $listContent = $('#lists');
		$listContent.empty();
		if (id == null || (''+id).length==0) {
			currentList = id;
			$('.statusbar.lists').hideStatus();
			$listContent.smartLoad(VCOM.server+'/listhome.ui',function(){
				VCOM.parse(this);
				
				$listContent.find('li.learningwords').on('click',function() {
					VCOM.showUserVocab('learning');
				}).end()
				.find('li.masteredwords').on('click',function() {
					VCOM.showUserVocab('mastered');
				}).end()
				.find('li.allwords').on('click',function() {
					VCOM.showUserVocab('all');
				});			
				if (typeof callback ==='function') callback.call(VCOM);
			});
			return true;
		} else {
			currentList = id;
			if (isNaN(parseInt(id))) {
				if (id in userVocabOptions) {
					VCOM.showUserVocab(id,callback);
					return true;
				}
			} else {
				$listContent.smartLoad(VCOM.server+'/list.ui',{id:parseInt(id)},function(){			
					$progress.find('span.title').text($(this).find('h1').text());
					updateListProgress();
					VCOM.parse(this);
					if (typeof callback ==='function') callback.call(VCOM);
				});
				return true;
			}
		}
	}
		
	VCOM.showUserVocab = function( type, callback ) {
		loaded=true;
		options = userVocabOptions[type] || userVocabOptions['learning'];
		VCOM.mode('lists');
		$lists = $('#lists').empty();
		updateListProgress();
		$.post(options.url,options.params,function(data) {
			$lists.append(renderList($.extend({ words: data.result.results, total: data.result.total },options)));
			$lists.find('.dynamictext').resizeText();
			$lists.drawProgress();
			updateListProgress();			
			if (typeof callback ==='function') callback.call(VCOM);
		}).fail(VCOM.errorHandler(function(){
			VCOM.showUserVocab(type,callback);
		}));
	};
	
	function updateListProgress() {
		
		if (VCOM.get('mode') != 'lists') return;
		
		var $listContent = $('#lists');
		var $list = $listContent.find('.wordlist');
		var data = $list.data(),
		    pdata = VCOM.playerData(),
		    currentList = null;
		if (pdata && pdata.lists && data && data.listId) {
			for (var i=0;i<pdata.lists.length;i++) {
				if (pdata.lists[i].listId==data.listId) {
					currentList = pdata.lists[i];
				}
			}
		}
		if (data && data.system) {
			statusBarProgress.hideStatus();
		} else if (currentList) {
			$list.addClass('learning');
			$listContent.drawProgress();
			$progress.find('i').attr('data-prg',parseInt(currentList.progress * 100));
			$progress.drawProgress();			
			statusBarProgress.showStatus();
			statusBarLearn.hideStatus();
			statusBarProgress.toggleClass('mastered',currentList.progress==1);						
		} else {			
			$list.removeClass('learning');
			if (data && data.learnable) {
				statusBarLearn.showStatus();
			} else {
				statusBarLearn.hideStatus();
			}
			statusBarProgress.hideStatus();
			statusBarProgress.removeClass('mastered');
		}	
	}
	
	function renderList(listData) {
		var list = $('<div class="wordlist">\
		<div class="centeredcontent">\
		<header>\
			<h1 class="title dynamictext"></h1>\
			<div class="wordcount"></div>\
		   	<div class="description"></div>\
	    </header>\
		<ol class="words"></ol>\
	    </div></div>').data(listData);
		list.find('.title').text(listData.title);
		
		var desc = list.find('.description').html(Format.htmlFormat(listData.description));
		if (listData.words.length == 0 && listData.empty) {
			desc.append('<p>'+listData.empty+'</p>');
		}
		
		
		list.find('ol').append( listData.words.map(renderItem) );
		if (listData.total>0) list.find('.wordcount').text(listData.total == 1 ? '1 word':listData.total+' words');
		if (listData.words.length < listData.total) {
			list.addClass('hasmore').one('fetch',function fetchMore() {
				var params = $.extend({start:list.find('li').length},listData.params);
				(function doIt(){
					$.post(listData.url,params,function(data) {
						if (data.result && data.result.results) {
							var items = data.result.results;
							var toAdd = $(items.map(renderItem));						
							items.forEach(VCOM.updateProgress);						
							list.find('ol').append(toAdd);
							toAdd.drawProgress().find('.dynamictext').resizeText();				
							if (data.result.results.length + params.start < data.result.total) {
								list.one('fetch',fetchMore);
							} else {
								list.removeClass('hasmore');
							}						
						}											
					}).fail(VCOM.errorHandler(doIt))
				})();
			});
		}
		list.addClass(listData['class']);
		listData.words.forEach(VCOM.updateProgress);		
		
		return list;
	}
	
	
	function renderItem(entry){
		var ret= $('<li class="entry learnable" >\
			<i class="progress"  />\
			<a class="word dynamictext" ></a>\
			<div class="definition"></div></li>');
		ret.find('i').attr('data-prg',parseInt( entry.prg* 100)).attr('data-word',entry.wrd);
		ret.find('a.word').text(entry.wrd).attr('href','/dictionary/'+entry.wrd);
		ret.find('.definition').text(entry.def);
		return ret[0];
	}
	
	
});