Module.define('vcom/urls', [ 'vcom/base' ], function( VCOM ) {

	var urlRe = /vocabcom:\/(\/[^?\/]*)([^?]*)(\?.*)?/
		
	var nextCommand = null;
	
	var commands = {
		'/play': function(cmd) {
			if (loggedIn()) {		
				VCOM.mode('play');
				return true;
			}
		},
		'/dictionary': function(cmd) {
			if (loggedIn()) {	
				var arg = cmd.argument ? cmd.argument.substring(1) : null;
				if (!arg || !arg.length) {
					VCOM.mode('dict');
				} else {
					VCOM.displayWord(arg);
				}
				return true;
			}
		},
		'/lists': function(cmd) {
			if (loggedIn()) {	
				VCOM.showList(cmd.argument ? cmd.argument.substring(1) : null);
				return true;
			}
		},
		'/logout': function(cmd) {
			VCOM.logout();
			return true;
		},
		'/login': function(cmd) {
			VCOM.login();
			return true;
		},
		'/loginkey': function(cmd) {
			VCOM.loginViaKey(cmd.query);
			return true;
		}
	};	
	
	commands['/dict']=commands['/dictionary'];
	commands['/list']=commands['/lists'];
			
	function openURL(url) {
		runCommand(parseCommand(url));
	}
	
	VCOM.openUrls = function() {
		window.handleOpenURL = openURL;
		if (window._urls) {
			if (nextCommand == null && window._urls.length) {
				var url =window._urls.pop();
				nextCommand = parseCommand(url);				
			}			
			delete window._urls;
		}
		if (nextCommand) {
			return runCommand(nextCommand);
		}		
	};
	
	function loggedIn() {
		var auth =  VCOM.get('auth');
		return auth && auth.loggedin;
	}
	
	function runCommand(cmd,timeout) {
		console.log('command queued');
		nextCommand = cmd;
		if (typeof nextCommand === 'function') {
			setTimeout(function(){
				if (cmd === nextCommand) {
					console.log('running command',cmd);
					if (cmd()) nextCommand = null;					
				}
			},timeout||10);
			return true;
		}		
	}
	
	function parseCommand(url) {
		var result = urlRe.exec(url);
		if (result) {			
			var args = { cmd: result[1],
			             argument: result[2],
			             query: parseQueryString(result[3]) };			
			var fn = commands[args.cmd];
			if (typeof fn==='function') {
				return function() {
					return fn(args);
				}
			};
		}
	}
	
	function parseQueryString( qs ) {
		var ret = {};
		if (typeof qs == 'string') {
			var args = qs.substring(1).split('&');
			for (var i=0;i<args.length;i++) {
				var pair = args[i].split('=');
				ret[ decodeURIComponent(pair[0]) ] = decodeURIComponent(pair[1]); 
			}
		}
		return ret;
	}
		
	return VCOM;
});	
