Module.define('vcom/vcom', ['vcom/base',
                            'jquery', 
                            'lru',
                            'spinner',
                            'gestures'], 
		function( VCOM, $, LRU ) {
	
	var isIOS = !!(/iphone|ipad|ipod/gi).test(window.navigator.appVersion);
	
	VCOM.tags = {
		'examples': 'vcom/examplebrowser',
		'wordfamily': 'vcom/wordfamily'
	};
	
	$('#loadmsg').text('starting');

	var progressCache = new LRU({ttl: 60 * 1000 * 60}); // 1 hour
		
	function updateCache(response) {
		if (response.prg && response.prg.length) {
			response.prg.forEach(VCOM.updateProgress);
		}
	}
	
	
	
	VCOM.back = function() {
		if (history.length) {
			history.pop()();	
			return true;
		} else {
			return false;
		}
	}
	
	VCOM.updateProgress = function(data) {
		if (data && data.wrd) progressCache.put(data.wrd,data);
	};
		
	VCOM.getProgress = function(word,callback) {
		var progress = progressCache.get(word);
		if (progress) {
			callback.call(this,progress);
		} else {
			$.post(VCOM.server+'/progress.json',{word:word},function(response){
				progressCache.put(word,response);
				callback.call(this,response);
			}).fail(VCOM.errorHandler(function(){
				VCOM.getProgress(word,callback);
			}));
		}			
	};
	
	VCOM.startLearning = function(word) {
		if (word) {
			if (typeof word != 'string') {
				if (word.getAttribute) word = word.getAttribute('word');
				else if (word.attr) word = word.attr('word');
			}		
			VCOM.status.message('<h3>Adding to your learning queue...</h3>',true);
			$.post(VCOM.server+'/startlearning.json',{word:word},function(status){
				if (status && status.success) {
					updateCache(status);
					$('body > .wrapper').trigger('vcom.learnword.start');					
				} else {
					VCOM.status.message('<h3>Unable to add word</h3>',false);
				}
			},'json').fail(VCOM.errorHandler(function(){
				VCOM.startLearning(word);
			}));
		} else {
			VCOM.status.message('<h3>Unable to add word</h3>',false);
		}
	};
	
	VCOM.stopLearning = function(word) {
		if (word) {
			if (typeof word != 'string') {
				if (word.getAttribute) word = word.getAttribute('word');
				else if (word.attr) word = word.attr('word');
			}
			VCOM.status.message('<h3>Changing word priority...</h3>',false);
			$.post(VCOM.server+'/stoplearning.json',{word:word},function(status){
				if (status && status.success) {					
					updateCache(status);
					$('body > .wrapper').trigger('vcom.learnword.stop');	
				} else {
					VCOM.status.message('<h3>Unable to remove word</h3>',false);
				}
			},'json').fail(VCOM.errorHandler(function(){
				VCOM.stopLearning(word);
			}));
		} else {
			VCOM.status.message('<h3>Unable to remove word</h3>',false);
		}
	};
		
	VCOM.search = function(search) {
		$('#search').val(search).trigger('change');	
	};
	
	VCOM.playAudio=function(node,pronunIdx) {
		if (!node) return;
		if (node.jquery) node=node[0];
		if (node.getAttribute) {
			var code = node.getAttribute('data-audio');
			if (code && code.length) {
				code = code.split(',');
				VCOM._playAudio(code[pronunIdx||0]);
			}
		}
	};
	
	var audio = null;
	
	VCOM._playAudio=VCOM._playAudio || function(code) {
		if (code && code.length) {
			/*$.doTimeout('audio',50,function(){
				var media = new Media('http://s3.amazonaws.com/audio.vocabulary.com/1.0/us/'+code+'.mp3');
				media.play();	
			});*/
			audio=new Audio('http://s3.amazonaws.com/audio.vocabulary.com/1.0/us/'+code+'.mp3');
			audio.play();				
		}
	};
					
	// MODE SWITCHING
	VCOM.mode = function(mode) {
		if (arguments.length==0) {
			return VCOM.get('mode');
		} else {
			VCOM.set('mode',mode);			
		}
		return this;
	};
	
	VCOM.on('change:mode',function(vcom,mode){
		var prev = VCOM.previous('mode');
		if (prev) {
			VCOM.history.push(function(){ VCOM.mode(prev); });
		}		
		$body = $('body');
		if (mode != 'dict') $('#search:focus').blur(); // ios bug workaround				
		$body.removeClass('mode-play mode-dict mode-lists')
		$('.mainNav ul li.selected').removeClass('selected');

		if (mode != null) {
		$body.addClass('mode-'+mode);		
		$('.mainNav ul li.'+mode+'Tab').addClass('selected');		
		window.localStorage.setItem('mode', mode);
		$('body > .wrapper').trigger({type:'vcom.switchmode',mode:mode});
		}
	});
		
	$.fn.resizeText = function() {		
		this.css('font-size','');		
		var sizes = [], remaining;
		do {
			remaining = this.filter(function(){ 
				var cw   = this.clientWidth,
				    size = parseFloat($(this).css('font-size')), 
				    sw   = this.scrollWidth;
				if (cw>0 && cw < sw && size > 1) {
					sizes.push(size-((size>14)?2:1));
					return true;
				}
			}).css('font-size',function(i){ 
				return sizes[i];
			});
			sizes=[];
		} while (remaining.length);
	};
	
	VCOM.parse = function(element) {
		element = element ? $(element) : $('body');
		for (var tag in VCOM.tags) {
			var elements = element.find('vcom\\:'+tag);
			if (elements.length) (function(tag,elements){
				Module.after( VCOM.tags[tag], function(mod){
				if (mod.construct) elements.each(mod.construct);
			});})(tag,elements);
		}
		element.find('.dynamictext').resizeText();
		element.drawProgress();
	};
		
	VCOM.on('login',function() {			
		history =[];
		$.doTimeout('window-resize',50,function(){
			$('.resizable').trigger('resized');
			$('.dynamictext').resizeText();	
		});
		setTimeout(function(){
			if (!VCOM.openUrls()) {
				if (window.debug) console.log('switching mode to',window.localStorage.mode||'play',VCOM.mode());
				VCOM.mode(window.localStorage.mode||'play');			
			}
		},10);
		
		if (isIOS) {
			// fix ios7 bug with focus on search box
			$('#search').on('touchstart',function(){
			    if (document.activeElement !== this) {
			    	this.focus();
			    	return false;
			    }
			});
		}
		
	});
		
	Module.after(['vcom/usermenu',
	              'vcom/status',
	              'vcom/challenge',
	              'vcom/dictionary',
	              'vcom/listtab',
	              'vcom/login',
	              'vcom/progress',
	              'vcom/examplebrowser',
	              'vcom/wordfamily',
	              'vcom/apprater',
	              'vcom/urls'],function(cookie) {	

		$('#loadmsg').text('starting');
		
		document.addEventListener('deviceready',function() {
			init();
			$('#loadmsg').text('authorizing');
			if (!VCOM.openUrls()) {
				VCOM.auth();	
			}
		},false);
	});
	        	     
	function init(){
		// hide the splash screen
		if (navigator.splashscreen) navigator.splashscreen.hide(); 
				
		$('#loadmsg').text('starting.');
				
		var $body = $('body');
						
		$body.on('click','a[href*=\'www.vocabulary.com/lists\'],a[href^=\'/lists/\']',function(e){
			var match = this.getAttribute('href').match(/(?:https?:\/\/www.vocabulary.com)?\/lists\/([^\/]*)/);
			if (match && VCOM.showList(match[1])) {
				e.preventDefault();
			}
		}).on('click','a[href^="/dictionary"]',function(e) {
			var href = /\/dictionary\/([^\/]*)(\/([^\/]*))?/.exec(this.getAttribute('href'));
			if (href && href.length) {				
				if (href[1] && href[2]==undefined) {
					VCOM.displayWord(href[1]);
				} else if (href[1] && href[3] && href[1].length==2) {
					VCOM.displayWord(href[3],href[1]);
				} else {
					VCOM.mode('dict');
				}
				$('#search:focus').blur();
				e.preventDefault();
			}
		}).on('click',"a[href^='/leaderboards/']",function(e) {
			VCOM.showModal($('<div class="leaderboard"></div>').smartLoad(VCOM.server+$(this).attr('href')));
			e.preventDefault();
		}).on('click',"a[href^='/achievements']",function(e) {
			VCOM.showModal($('<div class="achievements"></div>').smartLoad(VCOM.server+$(this).attr('href')));
			e.preventDefault();
		}).on('click','.lookup[word]',function(e) {
			VCOM.displayWord(this.getAttribute('word'),this.getAttribute('lang')||'en');	
			e.preventDefault();
		}).on('click','a[href="/"]',function(e) {
			VCOM.mode('play');
			e.preventDefault();
		}).on('click','.mainNav ul li.playTab',function(e) {			
			VCOM.mode('play');
		}).on('click','.mainNav ul li.dictTab',function(e) {			
			VCOM.mode('dict');			
			$('#search').focus();
		}).on('click','.mainNav ul li.listsTab',function(e) {			
			VCOM.mode('lists');
		}).on('click','i.progress[data-word]',function(e){
			VCOM.showWordProgress(this.getAttribute('data-word'));
			e.preventDefault();
		}).on('click','a[data-audio]',function(e){
			VCOM._playAudio(this.getAttribute('data-audio'));
			e.preventDefault();
		}).on('challenge.save',updateCache);
				
		Module.after(['shake'],function(shake){
			var shakeMonitor = new shake.Shake({ success: function(){ VCOM.randomWord(); }});		
			VCOM.on('change:mode',function() {
				var mode = VCOM.mode();
				if (mode==='dict') shakeMonitor.startWatch();
				else shakeMonitor.stopWatch();
			});
		});
		
		
		$(window).on('resize orientationchange',function(){ 			
			$.doTimeout('window-resize',50,function(){				
				$('.resizable').trigger('resized');
				$('.dynamictext').resizeText();	
			});			
		});
				
		// default click handler
		$('html').on('click','a[href]',function(e){
			if (!e.isDefaultPrevented()) {				
				var href = this.getAttribute('href');
				if (href) {
					if (href.indexOf('javascript:')==0 || href.indexOf('#')==0) return;				
					if (href.indexOf('http')!=0) href = 'http://www.vocabulary.com'+href;
					window.open(href, "_system");					
				}
				e.preventDefault();
			}
		});
		
		
	}
	
	
	return VCOM;
});	
