Module.define('vcom/wordfamily',[ 'jquery'],
 function( $ ) {
	var hasCanvas  =  !!document.createElement('canvas').getContext;
	
	this.construct = function() {
		$this = $(this);
		var ret = $this.data('wordfamily');
		if (!ret ) ret = $this.data('wordfamily', ret = new WordFamilyDisplay(this));	
		return ret;
	};
	
	this.WordFamilyDisplay = WordFamilyDisplay;

	function WordFamilyDisplay( element ) {
		
		element = $(element).addClass('vcom_wordfamily resizable').empty().on('resized',resized);
		var root = $('<ul></ul>').appendTo(element);
		var currentWord = element.attr("word");
		
		var totalWidth = element.innerWidth()-2;
		
		var tooltip = $('<div class="tooltip"><h3></h3><p></p><span class="nub"></span></div>')
		               .hide().appendTo(element)
		               .on('click',function(){
		            	   go(hoveredData);
		               });
		
		var hoveredData=null;
		
		element.on('mouseenter','.bar',function(){			
			var data = hoveredData =$(this).data();
			hover.call(this,data.word,data,data.ffreq);	
		}).on('mouseleave','.bar',function(){
			clearhover();			
		}).on('click','.bar',function(){
			go($(this).data());						
		});
		
		function go(data) {
			if (data) {
				var rw = data;
				while (!rw.hw && rw.parentWord != null) rw = rw.parentWord;
				if (rw.word === currentWord) {
					renderFamily(data.word);
					clearhover();
					element.trigger('wordfamily-select');
					var eb = element.closest('.wordPage').find('.vcom_examples').data('examplebrowser');
					if (eb != null) {
						eb.search(data.word);
					}
				} else {
					VCOM.displayWord(data.word);
				}
			}	
		}
				
		renderFamily(currentWord);
		
		function resized() {		
			root.hide();
			var w = element.innerWidth()-2;
			if (totalWidth != w) {				
				totalWidth = w;
				root.show();
				renderFamily(currentWord);		
			} else {
				root.show();
			}			
		}
				
		function renderFamily( currentWord ) {	
			var data = eval("("+element.attr("data")+")");			
			if (!data || !data.length) return;
			//element.attr('data',null);
			
			for (var i=0;i<data.length;i++) {
				if (data[i].word == currentWord) data[i].selected=true;
			}
			
			root.empty();
			var wordsByWord = {};
			var rootWord = null;
			var selectedWord = null;
			
			for (var i =0;i<data.length;i++) {
				wordsByWord[data[i].word] = data[i];
				data[i].children=[];
				if (data[i].selected) selectedWord = data[i];
			}			
			for (var i =0;i<data.length;i++) {
				var pw=data[i].parent;
				if (pw) {
					var p = wordsByWord[pw];
					data[i].parentWord = p;
					p.children.push(data[i]);
				} else {
					rootWord = data[i];
				}
			}
			
			var depth=0;
			for (var x=selectedWord;x!=null;x=x.parentWord) {
				depth++;
				x.inPath=true;
			}
			for (var x=selectedWord,i=depth;x!=null;x=x.parentWord,i--) x.depth=i;
			
			renderRecursive(selectedWord,depth);
			
			$('.bar span').each(function(){
				var e = $(this);
				var w = e.outerWidth();
				if (w+4 > e.parent().innerWidth()) {
					e.hide();
				} else {
					e.show().css('margin-left',(-w/2)+'px');
				}
			});
		}

		function renderRecursive(data,level) {						
			var rootContainer = $('<li></li>').data(data).appendTo(root);
			var scale = computeScale(data,totalWidth,totalWidth);
			var ret = renderAll(data,rootContainer,scale,level);
			var ctx = null;
			if (data.parentWord) {								
				var parent = renderRecursive(data.parentWord,level-1);				
				
				var canvas = $('<canvas></canvas>')
	               .attr('width',totalWidth)
	               .attr('height',20)
	               .appendTo(rootContainer)[0];
				if (canvas.getContext) {
					var ctx = canvas.getContext('2d');
					ctx.fillStyle='#ececec';
					ctx.beginPath();
					ctx.moveTo(0,0);
					ctx.lineTo(canvas.width,0);
					ctx.bezierCurveTo(canvas.width,canvas.height,
					          parent.offsetLeft+parent.clientWidth,0,
					          parent.offsetLeft+parent.clientWidth,canvas.height);
					ctx.lineTo(parent.offsetLeft,canvas.height);
					ctx.bezierCurveTo(parent.offsetLeft, 0,
									  0, canvas.height, 
					         		 0,0);		
					ctx.closePath();												
					ctx.fill();
				}				
			} else {
				var canvas = $('<canvas></canvas>')
	               .attr('width',totalWidth)
	               .attr('height',20).appendTo(rootContainer)[0];
				if (canvas.getContext) {
					var ctx = canvas.getContext('2d');
					ctx.fillStyle='#ececec';
					ctx.fillRect(0,0,canvas.width,canvas.height);
				}				
				root.append( $('<li></li>').append(
						$('<a class="bar needsclick"></a>')
							.css('width',totalWidth+'px')
							.data(data)
							.append($('<span></span>').text("the \""+data.word+"\" family"))
				));	
			}
			return ret;
		}
		
		function computeScale(data,width,target) {
			var cnt = data.children.length;			
			var scale = width / data.ffreq; 			
			var tw = Math.round(data.freq * scale);
			for (var i=0;i<data.children.length;i++) {
				tw+= Math.max(2, Math.round(scale * data.children[i].ffreq));
			}
			if (tw > target) {		
				return computeScale(data,width - (tw-width));
			}
			return scale;
		}
		
		function renderAll(data,container,scale,depth) {
			var ret=render(data,data.word,data.freq,container,scale,depth);			
			for (var i=0;i<data.children.length;i++) {
				if (data.children[i].type==1) {
					ret = render(data.children[i],data.children[i].word,data.children[i].ffreq,container,scale,depth) || ret;	
				}
			}
			for (var i=0;i<data.children.length;i++) {
				if (data.children[i].type>1) {
					ret = render(data.children[i],data.children[i].word,data.children[i].ffreq,container,scale,depth) || ret;	
				}
			}
			return ret;
		}
		
		function render(data,word,freq,container,scale,depth) {
			var h = Math.max(2, Math.round(scale * freq));
			if (freq==0 && data.depth == depth) h = totalWidth;
			var bar = $('<a></a>')
			          .addClass('needsclick bar type'+data.type)
			          .data(data)
			          .width(h)
			          .append($('<span></span>').text(word))
			          .appendTo(container);
			if (data.hw) bar.addClass('hw');			
			if (data.selected || (data.inPath && (data.depth == depth+1))) {
				bar.addClass('selected');
				if (data.selected && (data.depth == depth)) bar.addClass('current');			
				return bar[0];
			}
		}
		
		function clearhover() {
			tooltip.hide();		
		}
	
		function hover(word,data,freq) {
			var hovertext=null;			
			if (freq) {
				var ppo = formatNumber(1+parseInt(''+(1/(freq/4000)))); 				
				if (freq == data.freq) {
					hovertext = '\''+word+'\' will appear on average once every '+ppo+' pages.';
				} else {
					hovertext = 'All forms of \''+word+'\' will appear on average once every '+ppo+' pages.';
				}
			} else {
				hovertext = '\''+word+'\' is extremely rare.';
			}		
			tooltip.find('p').html(hovertext);
			tooltip.find('h3').text(word+':');
			
			var midx = this.offsetLeft + (this.clientWidth/2);
			var maxx = (this.offsetParent.clientWidth - tooltip.width())+10;
			var left = Math.min(Math.max(-10,midx - (tooltip.width() / 2)),maxx);
			var nub=tooltip.find('.nub');
			nub.css('left',(midx - left - (nub.width()/2))+'px');
			tooltip.css({'left': left+'px',
						 'bottom' : ((this.offsetParent.clientHeight-this.offsetTop)+10)+'px' }).show();
		}
		
		function formatNumber(n) {	
			 return n.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
		}						
	}
});